"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from polar_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union, cast


class Products(BaseSDK):
    def list(
        self,
        *,
        id: OptionalNullable[
            Union[
                models.QueryParamProductIDFilter,
                models.QueryParamProductIDFilterTypedDict,
            ]
        ] = UNSET,
        organization_id: OptionalNullable[
            Union[
                models.ProductsListQueryParamOrganizationIDFilter,
                models.ProductsListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        is_archived: OptionalNullable[bool] = UNSET,
        is_recurring: OptionalNullable[bool] = UNSET,
        benefit_id: OptionalNullable[
            Union[models.BenefitIDFilter, models.BenefitIDFilterTypedDict]
        ] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.ProductSortProperty]] = UNSET,
        metadata: OptionalNullable[
            Union[
                Dict[str, models.MetadataQuery],
                Dict[str, models.MetadataQueryTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ProductsListResponse]:
        r"""List Products

        List products.

        **Scopes**: `products:read` `products:write`

        :param id: Filter by product ID.
        :param organization_id: Filter by organization ID.
        :param query: Filter by product name.
        :param is_archived: Filter on archived products.
        :param is_recurring: Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned.
        :param benefit_id: Filter products granting specific benefit.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param metadata: Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsListRequest(
            id=id,
            organization_id=organization_id,
            query=query,
            is_archived=is_archived,
            is_recurring=is_recurring,
            benefit_id=benefit_id,
            page=page,
            limit=limit,
            sorting=sorting,
            metadata=metadata,
        )

        req = self._build_request(
            method="GET",
            path="/v1/products/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:list",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ProductsListResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                id=id,
                organization_id=organization_id,
                query=query,
                is_archived=is_archived,
                is_recurring=is_recurring,
                benefit_id=benefit_id,
                page=next_page,
                limit=limit,
                sorting=sorting,
                metadata=metadata,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ProductsListResponse(
                result=unmarshal_json_response(models.ListResourceProduct, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        id: OptionalNullable[
            Union[
                models.QueryParamProductIDFilter,
                models.QueryParamProductIDFilterTypedDict,
            ]
        ] = UNSET,
        organization_id: OptionalNullable[
            Union[
                models.ProductsListQueryParamOrganizationIDFilter,
                models.ProductsListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        is_archived: OptionalNullable[bool] = UNSET,
        is_recurring: OptionalNullable[bool] = UNSET,
        benefit_id: OptionalNullable[
            Union[models.BenefitIDFilter, models.BenefitIDFilterTypedDict]
        ] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.ProductSortProperty]] = UNSET,
        metadata: OptionalNullable[
            Union[
                Dict[str, models.MetadataQuery],
                Dict[str, models.MetadataQueryTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ProductsListResponse]:
        r"""List Products

        List products.

        **Scopes**: `products:read` `products:write`

        :param id: Filter by product ID.
        :param organization_id: Filter by organization ID.
        :param query: Filter by product name.
        :param is_archived: Filter on archived products.
        :param is_recurring: Filter on recurring products. If `true`, only subscriptions tiers are returned. If `false`, only one-time purchase products are returned.
        :param benefit_id: Filter products granting specific benefit.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param metadata: Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsListRequest(
            id=id,
            organization_id=organization_id,
            query=query,
            is_archived=is_archived,
            is_recurring=is_recurring,
            benefit_id=benefit_id,
            page=page,
            limit=limit,
            sorting=sorting,
            metadata=metadata,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/products/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:list",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ProductsListResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list(
                id=id,
                organization_id=organization_id,
                query=query,
                is_archived=is_archived,
                is_recurring=is_recurring,
                benefit_id=benefit_id,
                page=next_page,
                limit=limit,
                sorting=sorting,
                metadata=metadata,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.ProductsListResponse(
                result=unmarshal_json_response(models.ListResourceProduct, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def create(
        self,
        *,
        request: Union[models.ProductCreate, models.ProductCreateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Create Product

        Create a product.

        **Scopes**: `products:write`

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ProductCreate)
        request = cast(models.ProductCreate, request)

        req = self._build_request(
            method="POST",
            path="/v1/products/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ProductCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:create",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        request: Union[models.ProductCreate, models.ProductCreateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Create Product

        Create a product.

        **Scopes**: `products:write`

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.ProductCreate)
        request = cast(models.ProductCreate, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/products/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.ProductCreate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:create",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Get Product

        Get a product by ID.

        **Scopes**: `products:read` `products:write`

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsGetRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/products/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Get Product

        Get a product by ID.

        **Scopes**: `products:read` `products:write`

        :param id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsGetRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/products/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def update(
        self,
        *,
        id: str,
        product_update: Union[models.ProductUpdate, models.ProductUpdateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Update Product

        Update a product.

        **Scopes**: `products:write`

        :param id:
        :param product_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsUpdateRequest(
            id=id,
            product_update=utils.get_pydantic_model(
                product_update, models.ProductUpdate
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/v1/products/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.product_update, False, False, "json", models.ProductUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:update",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(models.NotPermittedData, http_res)
            raise models.NotPermitted(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        id: str,
        product_update: Union[models.ProductUpdate, models.ProductUpdateTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Update Product

        Update a product.

        **Scopes**: `products:write`

        :param id:
        :param product_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsUpdateRequest(
            id=id,
            product_update=utils.get_pydantic_model(
                product_update, models.ProductUpdate
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/v1/products/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.product_update, False, False, "json", models.ProductUpdate
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:update",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(models.NotPermittedData, http_res)
            raise models.NotPermitted(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def update_benefits(
        self,
        *,
        id: str,
        product_benefits_update: Union[
            models.ProductBenefitsUpdate, models.ProductBenefitsUpdateTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Update Product Benefits

        Update benefits granted by a product.

        **Scopes**: `products:write`

        :param id:
        :param product_benefits_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsUpdateBenefitsRequest(
            id=id,
            product_benefits_update=utils.get_pydantic_model(
                product_benefits_update, models.ProductBenefitsUpdate
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/products/{id}/benefits",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.product_benefits_update,
                False,
                False,
                "json",
                models.ProductBenefitsUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:update_benefits",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(models.NotPermittedData, http_res)
            raise models.NotPermitted(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def update_benefits_async(
        self,
        *,
        id: str,
        product_benefits_update: Union[
            models.ProductBenefitsUpdate, models.ProductBenefitsUpdateTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Product:
        r"""Update Product Benefits

        Update benefits granted by a product.

        **Scopes**: `products:write`

        :param id:
        :param product_benefits_update:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ProductsUpdateBenefitsRequest(
            id=id,
            product_benefits_update=utils.get_pydantic_model(
                product_benefits_update, models.ProductBenefitsUpdate
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/products/{id}/benefits",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.product_benefits_update,
                False,
                False,
                "json",
                models.ProductBenefitsUpdate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="products:update_benefits",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["403", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Product, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(models.NotPermittedData, http_res)
            raise models.NotPermitted(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .customfieldnumberproperties import (
    CustomFieldNumberProperties,
    CustomFieldNumberPropertiesTypedDict,
)
from .metadataoutputtype import MetadataOutputType, MetadataOutputTypeTypedDict
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from polar_sdk.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Dict, Literal
from typing_extensions import Annotated, TypedDict


class CustomFieldNumberTypedDict(TypedDict):
    r"""Schema for a custom field of type number."""

    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    metadata: Dict[str, MetadataOutputTypeTypedDict]
    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value."""
    name: str
    r"""Name of the custom field."""
    organization_id: str
    r"""The ID of the organization owning the custom field."""
    properties: CustomFieldNumberPropertiesTypedDict
    type: Literal["number"]


class CustomFieldNumber(BaseModel):
    r"""Schema for a custom field of type number."""

    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    metadata: Dict[str, MetadataOutputType]

    slug: str
    r"""Identifier of the custom field. It'll be used as key when storing the value."""

    name: str
    r"""Name of the custom field."""

    organization_id: str
    r"""The ID of the organization owning the custom field."""

    properties: CustomFieldNumberProperties

    TYPE: Annotated[
        Annotated[Literal["number"], AfterValidator(validate_const("number"))],
        pydantic.Field(alias="type"),
    ] = "number"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class DisputeStatus(str, Enum):
    PREVENTED = "prevented"
    EARLY_WARNING = "early_warning"
    NEEDS_RESPONSE = "needs_response"
    UNDER_REVIEW = "under_review"
    LOST = "lost"
    WON = "won"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorizationcodetokenrequest import (
    AuthorizationCodeTokenRequest,
    AuthorizationCodeTokenRequestTypedDict,
)
from .refreshtokenrequest import RefreshTokenRequest, RefreshTokenRequestTypedDict
from .webtokenrequest import WebTokenRequest, WebTokenRequestTypedDict
from typing import Union
from typing_extensions import TypeAliasType


Oauth2RequestTokenRequestBodyTypedDict = TypeAliasType(
    "Oauth2RequestTokenRequestBodyTypedDict",
    Union[
        RefreshTokenRequestTypedDict,
        AuthorizationCodeTokenRequestTypedDict,
        WebTokenRequestTypedDict,
    ],
)


Oauth2RequestTokenRequestBody = TypeAliasType(
    "Oauth2RequestTokenRequestBody",
    Union[RefreshTokenRequest, AuthorizationCodeTokenRequest, WebTokenRequest],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .eventsource import EventSource
from .eventtypessortproperty import EventTypesSortProperty
from .listresource_eventtypewithstats_ import (
    ListResourceEventTypeWithStats,
    ListResourceEventTypeWithStatsTypedDict,
)
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from polar_sdk.utils import FieldMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Callable, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


EventTypesListQueryParamOrganizationIDFilterTypedDict = TypeAliasType(
    "EventTypesListQueryParamOrganizationIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventTypesListQueryParamOrganizationIDFilter = TypeAliasType(
    "EventTypesListQueryParamOrganizationIDFilter", Union[str, List[str]]
)
r"""Filter by organization ID."""


EventTypesListQueryParamCustomerIDFilterTypedDict = TypeAliasType(
    "EventTypesListQueryParamCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventTypesListQueryParamCustomerIDFilter = TypeAliasType(
    "EventTypesListQueryParamCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by customer ID."""


EventTypesListQueryParamExternalCustomerIDFilterTypedDict = TypeAliasType(
    "EventTypesListQueryParamExternalCustomerIDFilterTypedDict", Union[str, List[str]]
)
r"""Filter by external customer ID."""


EventTypesListQueryParamExternalCustomerIDFilter = TypeAliasType(
    "EventTypesListQueryParamExternalCustomerIDFilter", Union[str, List[str]]
)
r"""Filter by external customer ID."""


class EventTypesListRequestTypedDict(TypedDict):
    organization_id: NotRequired[
        Nullable[EventTypesListQueryParamOrganizationIDFilterTypedDict]
    ]
    r"""Filter by organization ID."""
    customer_id: NotRequired[
        Nullable[EventTypesListQueryParamCustomerIDFilterTypedDict]
    ]
    r"""Filter by customer ID."""
    external_customer_id: NotRequired[
        Nullable[EventTypesListQueryParamExternalCustomerIDFilterTypedDict]
    ]
    r"""Filter by external customer ID."""
    query: NotRequired[Nullable[str]]
    r"""Query to filter event types by name or label."""
    root_events: NotRequired[bool]
    r"""When true, only return event types with root events (parent_id IS NULL)."""
    parent_id: NotRequired[Nullable[str]]
    r"""Filter by specific parent event ID."""
    source: NotRequired[Nullable[EventSource]]
    r"""Filter by event source (system or user)."""
    page: NotRequired[int]
    r"""Page number, defaults to 1."""
    limit: NotRequired[int]
    r"""Size of a page, defaults to 10. Maximum is 100."""
    sorting: NotRequired[Nullable[List[EventTypesSortProperty]]]
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""


class EventTypesListRequest(BaseModel):
    organization_id: Annotated[
        OptionalNullable[EventTypesListQueryParamOrganizationIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by organization ID."""

    customer_id: Annotated[
        OptionalNullable[EventTypesListQueryParamCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by customer ID."""

    external_customer_id: Annotated[
        OptionalNullable[EventTypesListQueryParamExternalCustomerIDFilter],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by external customer ID."""

    query: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Query to filter event types by name or label."""

    root_events: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""When true, only return event types with root events (parent_id IS NULL)."""

    parent_id: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by specific parent event ID."""

    source: Annotated[
        OptionalNullable[EventSource],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Filter by event source (system or user)."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1
    r"""Page number, defaults to 1."""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 10
    r"""Size of a page, defaults to 10. Maximum is 100."""

    sorting: Annotated[
        OptionalNullable[List[EventTypesSortProperty]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "organization_id",
            "customer_id",
            "external_customer_id",
            "query",
            "root_events",
            "parent_id",
            "source",
            "page",
            "limit",
            "sorting",
        ]
        nullable_fields = [
            "organization_id",
            "customer_id",
            "external_customer_id",
            "query",
            "parent_id",
            "source",
            "sorting",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class EventTypesListResponseTypedDict(TypedDict):
    result: ListResourceEventTypeWithStatsTypedDict


class EventTypesListResponse(BaseModel):
    next: Callable[[], Optional[EventTypesListResponse]]

    result: ListResourceEventTypeWithStats



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .organizationcustomerportalsettings import (
    OrganizationCustomerPortalSettings,
    OrganizationCustomerPortalSettingsTypedDict,
)
from .subscriptionprorationbehavior import SubscriptionProrationBehavior
from datetime import datetime
from polar_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing_extensions import TypedDict


class CustomerOrganizationTypedDict(TypedDict):
    created_at: datetime
    r"""Creation timestamp of the object."""
    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""
    id: str
    r"""The ID of the object."""
    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""
    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""
    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""
    proration_behavior: SubscriptionProrationBehavior
    allow_customer_updates: bool
    r"""Whether customers can update their subscriptions from the customer portal."""
    customer_portal_settings: OrganizationCustomerPortalSettingsTypedDict


class CustomerOrganization(BaseModel):
    created_at: datetime
    r"""Creation timestamp of the object."""

    modified_at: Nullable[datetime]
    r"""Last modification timestamp of the object."""

    id: str
    r"""The ID of the object."""

    name: str
    r"""Organization name shown in checkout, customer portal, emails etc."""

    slug: str
    r"""Unique organization slug in checkout, customer portal and credit card statements."""

    avatar_url: Nullable[str]
    r"""Avatar URL shown in checkout, customer portal, emails etc."""

    proration_behavior: SubscriptionProrationBehavior

    allow_customer_updates: bool
    r"""Whether customers can update their subscriptions from the customer portal."""

    customer_portal_settings: OrganizationCustomerPortalSettings

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["modified_at", "avatar_url"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrganizationSortProperty(str, Enum):
    CREATED_AT = "created_at"
    MINUS_CREATED_AT = "-created_at"
    SLUG = "slug"
    MINUS_SLUG = "-slug"
    NAME = "name"
    MINUS_NAME = "-name"
    NEXT_REVIEW_THRESHOLD = "next_review_threshold"
    MINUS_NEXT_REVIEW_THRESHOLD = "-next_review_threshold"
    DAYS_IN_STATUS = "days_in_status"
    MINUS_DAYS_IN_STATUS = "-days_in_status"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ProductPriceSource(str, Enum):
    CATALOG = "catalog"
    AD_HOC = "ad_hoc"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BalanceOrderMetadataTypedDict(TypedDict):
    transaction_id: str
    order_id: str
    amount: int
    currency: str
    presentment_amount: int
    presentment_currency: str
    tax_amount: int
    fee: int
    product_id: NotRequired[str]
    subscription_id: NotRequired[str]
    tax_state: NotRequired[Nullable[str]]
    tax_country: NotRequired[Nullable[str]]


class BalanceOrderMetadata(BaseModel):
    transaction_id: str

    order_id: str

    amount: int

    currency: str

    presentment_amount: int

    presentment_currency: str

    tax_amount: int

    fee: int

    product_id: Optional[str] = None

    subscription_id: Optional[str] = None

    tax_state: OptionalNullable[str] = UNSET

    tax_country: OptionalNullable[str] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["product_id", "subscription_id", "tax_state", "tax_country"]
        nullable_fields = ["tax_state", "tax_country"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .memberrole import MemberRole
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MemberCreateTypedDict(TypedDict):
    r"""Schema for creating a new member."""

    customer_id: str
    r"""The ID of the customer this member belongs to."""
    email: str
    r"""The email address of the member."""
    name: NotRequired[Nullable[str]]
    external_id: NotRequired[Nullable[str]]
    r"""The ID of the member in your system. This must be unique within the customer."""
    role: NotRequired[MemberRole]


class MemberCreate(BaseModel):
    r"""Schema for creating a new member."""

    customer_id: str
    r"""The ID of the customer this member belongs to."""

    email: str
    r"""The email address of the member."""

    name: OptionalNullable[str] = UNSET

    external_id: OptionalNullable[str] = UNSET
    r"""The ID of the member in your system. This must be unique within the customer."""

    role: Optional[MemberRole] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["name", "external_id", "role"]
        nullable_fields = ["name", "external_id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookFormat(str, Enum):
    RAW = "raw"
    DISCORD = "discord"
    SLACK = "slack"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .subscriptioncreatecustomer import (
    SubscriptionCreateCustomer,
    SubscriptionCreateCustomerTypedDict,
)
from .subscriptioncreateexternalcustomer import (
    SubscriptionCreateExternalCustomer,
    SubscriptionCreateExternalCustomerTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


SubscriptionsCreateSubscriptionCreateTypedDict = TypeAliasType(
    "SubscriptionsCreateSubscriptionCreateTypedDict",
    Union[
        SubscriptionCreateCustomerTypedDict, SubscriptionCreateExternalCustomerTypedDict
    ],
)


SubscriptionsCreateSubscriptionCreate = TypeAliasType(
    "SubscriptionsCreateSubscriptionCreate",
    Union[SubscriptionCreateCustomer, SubscriptionCreateExternalCustomer],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from jsonpath import JSONPath
from polar_sdk import models, utils
from polar_sdk._hooks import HookContext
from polar_sdk.types import BaseModel, OptionalNullable, UNSET
from polar_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union, cast


class Events(BaseSDK):
    def list(
        self,
        *,
        filter_: OptionalNullable[str] = UNSET,
        start_timestamp: OptionalNullable[datetime] = UNSET,
        end_timestamp: OptionalNullable[datetime] = UNSET,
        organization_id: OptionalNullable[
            Union[
                models.EventsListQueryParamOrganizationIDFilter,
                models.EventsListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.EventsListQueryParamCustomerIDFilter,
                models.EventsListQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        external_customer_id: OptionalNullable[
            Union[
                models.QueryParamExternalCustomerIDFilter,
                models.QueryParamExternalCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        meter_id: OptionalNullable[str] = UNSET,
        name: OptionalNullable[
            Union[models.NameFilter, models.NameFilterTypedDict]
        ] = UNSET,
        source: OptionalNullable[
            Union[models.SourceFilter, models.SourceFilterTypedDict]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        parent_id: OptionalNullable[str] = UNSET,
        depth: OptionalNullable[int] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.EventSortProperty]] = UNSET,
        metadata: OptionalNullable[
            Union[
                Dict[str, models.MetadataQuery],
                Dict[str, models.MetadataQueryTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsListResponseEventsList:
        r"""List Events

        List events.

        **Scopes**: `events:read` `events:write`

        :param filter_: Filter events following filter clauses. JSON string following the same schema a meter filter clause.
        :param start_timestamp: Filter events after this timestamp.
        :param end_timestamp: Filter events before this timestamp.
        :param organization_id: Filter by organization ID.
        :param customer_id: Filter by customer ID.
        :param external_customer_id: Filter by external customer ID.
        :param meter_id: Filter by a meter filter clause.
        :param name: Filter by event name.
        :param source: Filter by event source.
        :param query: Query to filter events.
        :param parent_id: When combined with depth, use this event as the anchor instead of root events.
        :param depth: Fetch descendants up to this depth. When set: 0=root events only, 1=roots+children, etc. Max 5. When not set, returns all events.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param metadata: Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsListRequest(
            filter_=filter_,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            organization_id=organization_id,
            customer_id=customer_id,
            external_customer_id=external_customer_id,
            meter_id=meter_id,
            name=name,
            source=source,
            query=query,
            parent_id=parent_id,
            depth=depth,
            page=page,
            limit=limit,
            sorting=sorting,
            metadata=metadata,
        )

        req = self._build_request(
            method="GET",
            path="/v1/events/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:list",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EventsListResponseEventsList, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        filter_: OptionalNullable[str] = UNSET,
        start_timestamp: OptionalNullable[datetime] = UNSET,
        end_timestamp: OptionalNullable[datetime] = UNSET,
        organization_id: OptionalNullable[
            Union[
                models.EventsListQueryParamOrganizationIDFilter,
                models.EventsListQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.EventsListQueryParamCustomerIDFilter,
                models.EventsListQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        external_customer_id: OptionalNullable[
            Union[
                models.QueryParamExternalCustomerIDFilter,
                models.QueryParamExternalCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        meter_id: OptionalNullable[str] = UNSET,
        name: OptionalNullable[
            Union[models.NameFilter, models.NameFilterTypedDict]
        ] = UNSET,
        source: OptionalNullable[
            Union[models.SourceFilter, models.SourceFilterTypedDict]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        parent_id: OptionalNullable[str] = UNSET,
        depth: OptionalNullable[int] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.EventSortProperty]] = UNSET,
        metadata: OptionalNullable[
            Union[
                Dict[str, models.MetadataQuery],
                Dict[str, models.MetadataQueryTypedDict],
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsListResponseEventsList:
        r"""List Events

        List events.

        **Scopes**: `events:read` `events:write`

        :param filter_: Filter events following filter clauses. JSON string following the same schema a meter filter clause.
        :param start_timestamp: Filter events after this timestamp.
        :param end_timestamp: Filter events before this timestamp.
        :param organization_id: Filter by organization ID.
        :param customer_id: Filter by customer ID.
        :param external_customer_id: Filter by external customer ID.
        :param meter_id: Filter by a meter filter clause.
        :param name: Filter by event name.
        :param source: Filter by event source.
        :param query: Query to filter events.
        :param parent_id: When combined with depth, use this event as the anchor instead of root events.
        :param depth: Fetch descendants up to this depth. When set: 0=root events only, 1=roots+children, etc. Max 5. When not set, returns all events.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param metadata: Filter by metadata key-value pairs. It uses the `deepObject` style, e.g. `?metadata[key]=value`.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsListRequest(
            filter_=filter_,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            organization_id=organization_id,
            customer_id=customer_id,
            external_customer_id=external_customer_id,
            meter_id=meter_id,
            name=name,
            source=source,
            query=query,
            parent_id=parent_id,
            depth=depth,
            page=page,
            limit=limit,
            sorting=sorting,
            metadata=metadata,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/events/",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:list",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.EventsListResponseEventsList, http_res
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def list_names(
        self,
        *,
        organization_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamOrganizationIDFilter,
                models.EventsListNamesQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamCustomerIDFilter,
                models.EventsListNamesQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        external_customer_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamExternalCustomerIDFilter,
                models.EventsListNamesQueryParamExternalCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        source: OptionalNullable[
            Union[models.QueryParamSourceFilter, models.QueryParamSourceFilterTypedDict]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.EventNamesSortProperty]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.EventsListNamesResponse]:
        r"""List Event Names

        List event names.

        **Scopes**: `events:read` `events:write`

        :param organization_id: Filter by organization ID.
        :param customer_id: Filter by customer ID.
        :param external_customer_id: Filter by external customer ID.
        :param source: Filter by event source.
        :param query: Query to filter event names.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsListNamesRequest(
            organization_id=organization_id,
            customer_id=customer_id,
            external_customer_id=external_customer_id,
            source=source,
            query=query,
            page=page,
            limit=limit,
            sorting=sorting,
        )

        req = self._build_request(
            method="GET",
            path="/v1/events/names",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:list_names",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.EventsListNamesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list_names(
                organization_id=organization_id,
                customer_id=customer_id,
                external_customer_id=external_customer_id,
                source=source,
                query=query,
                page=next_page,
                limit=limit,
                sorting=sorting,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EventsListNamesResponse(
                result=unmarshal_json_response(models.ListResourceEventName, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def list_names_async(
        self,
        *,
        organization_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamOrganizationIDFilter,
                models.EventsListNamesQueryParamOrganizationIDFilterTypedDict,
            ]
        ] = UNSET,
        customer_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamCustomerIDFilter,
                models.EventsListNamesQueryParamCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        external_customer_id: OptionalNullable[
            Union[
                models.EventsListNamesQueryParamExternalCustomerIDFilter,
                models.EventsListNamesQueryParamExternalCustomerIDFilterTypedDict,
            ]
        ] = UNSET,
        source: OptionalNullable[
            Union[models.QueryParamSourceFilter, models.QueryParamSourceFilterTypedDict]
        ] = UNSET,
        query: OptionalNullable[str] = UNSET,
        page: Optional[int] = 1,
        limit: Optional[int] = 10,
        sorting: OptionalNullable[List[models.EventNamesSortProperty]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.EventsListNamesResponse]:
        r"""List Event Names

        List event names.

        **Scopes**: `events:read` `events:write`

        :param organization_id: Filter by organization ID.
        :param customer_id: Filter by customer ID.
        :param external_customer_id: Filter by external customer ID.
        :param source: Filter by event source.
        :param query: Query to filter event names.
        :param page: Page number, defaults to 1.
        :param limit: Size of a page, defaults to 10. Maximum is 100.
        :param sorting: Sorting criterion. Several criteria can be used simultaneously and will be applied in order. Add a minus sign `-` before the criteria name to sort by descending order.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsListNamesRequest(
            organization_id=organization_id,
            customer_id=customer_id,
            external_customer_id=external_customer_id,
            source=source,
            query=query,
            page=page,
            limit=limit,
            sorting=sorting,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/events/names",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:list_names",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.EventsListNamesResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            page = request.page if not request.page is None else 1
            next_page = page + 1

            num_pages = JSONPath("$.pagination.max_page").parse(body)
            if len(num_pages) == 0 or num_pages[0] <= page:
                return None

            if not http_res.text:
                return None
            results = JSONPath("$.items").parse(body)
            if len(results) == 0 or len(results[0]) == 0:
                return None
            limit = request.limit if not request.limit is None else 10
            if len(results[0]) < limit:
                return None

            return self.list_names(
                organization_id=organization_id,
                customer_id=customer_id,
                external_customer_id=external_customer_id,
                source=source,
                query=query,
                page=next_page,
                limit=limit,
                sorting=sorting,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.EventsListNamesResponse(
                result=unmarshal_json_response(models.ListResourceEventName, http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Event:
        r"""Get Event

        Get an event by ID.

        **Scopes**: `events:read` `events:write`

        :param id: The event ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsGetRequest(
            id=id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/events/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Event, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Event:
        r"""Get Event

        Get an event by ID.

        **Scopes**: `events:read` `events:write`

        :param id: The event ID.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.EventsGetRequest(
            id=id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/events/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.Event, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.ResourceNotFoundData, http_res
            )
            raise models.ResourceNotFound(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def ingest(
        self,
        *,
        request: Union[models.EventsIngest, models.EventsIngestTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsIngestResponse:
        r"""Ingest Events

        Ingest batch of events.

        **Scopes**: `events:write`

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.EventsIngest)
        request = cast(models.EventsIngest, request)

        req = self._build_request(
            method="POST",
            path="/v1/events/ingest",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.EventsIngest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:ingest",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.EventsIngestResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def ingest_async(
        self,
        *,
        request: Union[models.EventsIngest, models.EventsIngestTypedDict],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EventsIngestResponse:
        r"""Ingest Events

        Ingest batch of events.

        **Scopes**: `events:write`

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(request, models.EventsIngest)
        request = cast(models.EventsIngest, request)

        req = self._build_request_async(
            method="POST",
            path="/v1/events/ingest",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.EventsIngest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="events:ingest",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.EventsIngestResponse, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.HTTPValidationErrorData, http_res
            )
            raise models.HTTPValidationError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


MetricsTotalsOrdersTypedDict = TypeAliasType(
    "MetricsTotalsOrdersTypedDict", Union[int, float]
)


MetricsTotalsOrders = TypeAliasType("MetricsTotalsOrders", Union[int, float])


MetricsTotalsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRevenue = TypeAliasType("MetricsTotalsRevenue", Union[int, float])


MetricsTotalsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsNetRevenue = TypeAliasType("MetricsTotalsNetRevenue", Union[int, float])


MetricsTotalsCumulativeRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCumulativeRevenueTypedDict", Union[int, float]
)


MetricsTotalsCumulativeRevenue = TypeAliasType(
    "MetricsTotalsCumulativeRevenue", Union[int, float]
)


MetricsTotalsNetCumulativeRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNetCumulativeRevenueTypedDict", Union[int, float]
)


MetricsTotalsNetCumulativeRevenue = TypeAliasType(
    "MetricsTotalsNetCumulativeRevenue", Union[int, float]
)


MetricsTotalsCostsTypedDict = TypeAliasType(
    "MetricsTotalsCostsTypedDict", Union[int, float]
)


MetricsTotalsCosts = TypeAliasType("MetricsTotalsCosts", Union[int, float])


MetricsTotalsCumulativeCostsTypedDict = TypeAliasType(
    "MetricsTotalsCumulativeCostsTypedDict", Union[int, float]
)


MetricsTotalsCumulativeCosts = TypeAliasType(
    "MetricsTotalsCumulativeCosts", Union[int, float]
)


MetricsTotalsAverageOrderValueTypedDict = TypeAliasType(
    "MetricsTotalsAverageOrderValueTypedDict", Union[int, float]
)


MetricsTotalsAverageOrderValue = TypeAliasType(
    "MetricsTotalsAverageOrderValue", Union[int, float]
)


MetricsTotalsNetAverageOrderValueTypedDict = TypeAliasType(
    "MetricsTotalsNetAverageOrderValueTypedDict", Union[int, float]
)


MetricsTotalsNetAverageOrderValue = TypeAliasType(
    "MetricsTotalsNetAverageOrderValue", Union[int, float]
)


MetricsTotalsAverageRevenuePerUserTypedDict = TypeAliasType(
    "MetricsTotalsAverageRevenuePerUserTypedDict", Union[int, float]
)


MetricsTotalsAverageRevenuePerUser = TypeAliasType(
    "MetricsTotalsAverageRevenuePerUser", Union[int, float]
)


MetricsTotalsCostPerUserTypedDict = TypeAliasType(
    "MetricsTotalsCostPerUserTypedDict", Union[int, float]
)


MetricsTotalsCostPerUser = TypeAliasType("MetricsTotalsCostPerUser", Union[int, float])


MetricsTotalsActiveUserByEventTypedDict = TypeAliasType(
    "MetricsTotalsActiveUserByEventTypedDict", Union[int, float]
)


MetricsTotalsActiveUserByEvent = TypeAliasType(
    "MetricsTotalsActiveUserByEvent", Union[int, float]
)


MetricsTotalsOneTimeProductsTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProducts = TypeAliasType(
    "MetricsTotalsOneTimeProducts", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenueTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProductsRevenue = TypeAliasType(
    "MetricsTotalsOneTimeProductsRevenue", Union[int, float]
)


MetricsTotalsOneTimeProductsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsOneTimeProductsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsOneTimeProductsNetRevenue = TypeAliasType(
    "MetricsTotalsOneTimeProductsNetRevenue", Union[int, float]
)


MetricsTotalsNewSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptions = TypeAliasType(
    "MetricsTotalsNewSubscriptions", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsNewSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsNewSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsNewSubscriptionsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsNewSubscriptionsNetRevenue = TypeAliasType(
    "MetricsTotalsNewSubscriptionsNetRevenue", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptions = TypeAliasType(
    "MetricsTotalsRenewedSubscriptions", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenueTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsRevenue = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsRevenue", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict", Union[int, float]
)


MetricsTotalsRenewedSubscriptionsNetRevenue = TypeAliasType(
    "MetricsTotalsRenewedSubscriptionsNetRevenue", Union[int, float]
)


MetricsTotalsActiveSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsActiveSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsActiveSubscriptions = TypeAliasType(
    "MetricsTotalsActiveSubscriptions", Union[int, float]
)


MetricsTotalsCommittedSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsCommittedSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsCommittedSubscriptions = TypeAliasType(
    "MetricsTotalsCommittedSubscriptions", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict", Union[int, float]
)


MetricsTotalsCommittedMonthlyRecurringRevenue = TypeAliasType(
    "MetricsTotalsCommittedMonthlyRecurringRevenue", Union[int, float]
)


MetricsTotalsCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsCheckouts = TypeAliasType("MetricsTotalsCheckouts", Union[int, float])


MetricsTotalsSucceededCheckoutsTypedDict = TypeAliasType(
    "MetricsTotalsSucceededCheckoutsTypedDict", Union[int, float]
)


MetricsTotalsSucceededCheckouts = TypeAliasType(
    "MetricsTotalsSucceededCheckouts", Union[int, float]
)


MetricsTotalsCheckoutsConversionTypedDict = TypeAliasType(
    "MetricsTotalsCheckoutsConversionTypedDict", Union[int, float]
)


MetricsTotalsCheckoutsConversion = TypeAliasType(
    "MetricsTotalsCheckoutsConversion", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptions = TypeAliasType(
    "MetricsTotalsCanceledSubscriptions", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsCustomerService = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsCustomerService", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsLowQualityTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsLowQualityTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsLowQuality = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsLowQuality", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsMissingFeatures = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsMissingFeatures", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsSwitchedService = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsSwitchedService", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooComplexTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooComplexTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooComplex = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooComplex", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsTooExpensive = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsTooExpensive", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsUnusedTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsUnusedTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsUnused = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsUnused", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsOtherTypedDict = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsOtherTypedDict", Union[int, float]
)


MetricsTotalsCanceledSubscriptionsOther = TypeAliasType(
    "MetricsTotalsCanceledSubscriptionsOther", Union[int, float]
)


MetricsTotalsChurnedSubscriptionsTypedDict = TypeAliasType(
    "MetricsTotalsChurnedSubscriptionsTypedDict", Union[int, float]
)


MetricsTotalsChurnedSubscriptions = TypeAliasType(
    "MetricsTotalsChurnedSubscriptions", Union[int, float]
)


MetricsTotalsChurnRateTypedDict = TypeAliasType(
    "MetricsTotalsChurnRateTypedDict", Union[int, float]
)


MetricsTotalsChurnRate = TypeAliasType("MetricsTotalsChurnRate", Union[int, float])


MetricsTotalsLtvTypedDict = TypeAliasType(
    "MetricsTotalsLtvTypedDict", Union[int, float]
)


MetricsTotalsLtv = TypeAliasType("MetricsTotalsLtv", Union[int, float])


MetricsTotalsGrossMarginTypedDict = TypeAliasType(
    "MetricsTotalsGrossMarginTypedDict", Union[int, float]
)


MetricsTotalsGrossMargin = TypeAliasType("MetricsTotalsGrossMargin", Union[int, float])


MetricsTotalsGrossMarginPercentageTypedDict = TypeAliasType(
    "MetricsTotalsGrossMarginPercentageTypedDict", Union[int, float]
)


MetricsTotalsGrossMarginPercentage = TypeAliasType(
    "MetricsTotalsGrossMarginPercentage", Union[int, float]
)


MetricsTotalsCashflowTypedDict = TypeAliasType(
    "MetricsTotalsCashflowTypedDict", Union[int, float]
)


MetricsTotalsCashflow = TypeAliasType("MetricsTotalsCashflow", Union[int, float])


class MetricsTotalsTypedDict(TypedDict):
    orders: NotRequired[Nullable[MetricsTotalsOrdersTypedDict]]
    revenue: NotRequired[Nullable[MetricsTotalsRevenueTypedDict]]
    net_revenue: NotRequired[Nullable[MetricsTotalsNetRevenueTypedDict]]
    cumulative_revenue: NotRequired[Nullable[MetricsTotalsCumulativeRevenueTypedDict]]
    net_cumulative_revenue: NotRequired[
        Nullable[MetricsTotalsNetCumulativeRevenueTypedDict]
    ]
    costs: NotRequired[Nullable[MetricsTotalsCostsTypedDict]]
    cumulative_costs: NotRequired[Nullable[MetricsTotalsCumulativeCostsTypedDict]]
    average_order_value: NotRequired[Nullable[MetricsTotalsAverageOrderValueTypedDict]]
    net_average_order_value: NotRequired[
        Nullable[MetricsTotalsNetAverageOrderValueTypedDict]
    ]
    average_revenue_per_user: NotRequired[
        Nullable[MetricsTotalsAverageRevenuePerUserTypedDict]
    ]
    cost_per_user: NotRequired[Nullable[MetricsTotalsCostPerUserTypedDict]]
    active_user_by_event: NotRequired[Nullable[MetricsTotalsActiveUserByEventTypedDict]]
    one_time_products: NotRequired[Nullable[MetricsTotalsOneTimeProductsTypedDict]]
    one_time_products_revenue: NotRequired[
        Nullable[MetricsTotalsOneTimeProductsRevenueTypedDict]
    ]
    one_time_products_net_revenue: NotRequired[
        Nullable[MetricsTotalsOneTimeProductsNetRevenueTypedDict]
    ]
    new_subscriptions: NotRequired[Nullable[MetricsTotalsNewSubscriptionsTypedDict]]
    new_subscriptions_revenue: NotRequired[
        Nullable[MetricsTotalsNewSubscriptionsRevenueTypedDict]
    ]
    new_subscriptions_net_revenue: NotRequired[
        Nullable[MetricsTotalsNewSubscriptionsNetRevenueTypedDict]
    ]
    renewed_subscriptions: NotRequired[
        Nullable[MetricsTotalsRenewedSubscriptionsTypedDict]
    ]
    renewed_subscriptions_revenue: NotRequired[
        Nullable[MetricsTotalsRenewedSubscriptionsRevenueTypedDict]
    ]
    renewed_subscriptions_net_revenue: NotRequired[
        Nullable[MetricsTotalsRenewedSubscriptionsNetRevenueTypedDict]
    ]
    active_subscriptions: NotRequired[
        Nullable[MetricsTotalsActiveSubscriptionsTypedDict]
    ]
    committed_subscriptions: NotRequired[
        Nullable[MetricsTotalsCommittedSubscriptionsTypedDict]
    ]
    monthly_recurring_revenue: NotRequired[
        Nullable[MetricsTotalsMonthlyRecurringRevenueTypedDict]
    ]
    committed_monthly_recurring_revenue: NotRequired[
        Nullable[MetricsTotalsCommittedMonthlyRecurringRevenueTypedDict]
    ]
    checkouts: NotRequired[Nullable[MetricsTotalsCheckoutsTypedDict]]
    succeeded_checkouts: NotRequired[Nullable[MetricsTotalsSucceededCheckoutsTypedDict]]
    checkouts_conversion: NotRequired[
        Nullable[MetricsTotalsCheckoutsConversionTypedDict]
    ]
    canceled_subscriptions: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsTypedDict]
    ]
    canceled_subscriptions_customer_service: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsCustomerServiceTypedDict]
    ]
    canceled_subscriptions_low_quality: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsLowQualityTypedDict]
    ]
    canceled_subscriptions_missing_features: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsMissingFeaturesTypedDict]
    ]
    canceled_subscriptions_switched_service: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsSwitchedServiceTypedDict]
    ]
    canceled_subscriptions_too_complex: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsTooComplexTypedDict]
    ]
    canceled_subscriptions_too_expensive: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsTooExpensiveTypedDict]
    ]
    canceled_subscriptions_unused: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsUnusedTypedDict]
    ]
    canceled_subscriptions_other: NotRequired[
        Nullable[MetricsTotalsCanceledSubscriptionsOtherTypedDict]
    ]
    churned_subscriptions: NotRequired[
        Nullable[MetricsTotalsChurnedSubscriptionsTypedDict]
    ]
    churn_rate: NotRequired[Nullable[MetricsTotalsChurnRateTypedDict]]
    ltv: NotRequired[Nullable[MetricsTotalsLtvTypedDict]]
    gross_margin: NotRequired[Nullable[MetricsTotalsGrossMarginTypedDict]]
    gross_margin_percentage: NotRequired[
        Nullable[MetricsTotalsGrossMarginPercentageTypedDict]
    ]
    cashflow: NotRequired[Nullable[MetricsTotalsCashflowTypedDict]]


class MetricsTotals(BaseModel):
    orders: OptionalNullable[MetricsTotalsOrders] = UNSET

    revenue: OptionalNullable[MetricsTotalsRevenue] = UNSET

    net_revenue: OptionalNullable[MetricsTotalsNetRevenue] = UNSET

    cumulative_revenue: OptionalNullable[MetricsTotalsCumulativeRevenue] = UNSET

    net_cumulative_revenue: OptionalNullable[MetricsTotalsNetCumulativeRevenue] = UNSET

    costs: OptionalNullable[MetricsTotalsCosts] = UNSET

    cumulative_costs: OptionalNullable[MetricsTotalsCumulativeCosts] = UNSET

    average_order_value: OptionalNullable[MetricsTotalsAverageOrderValue] = UNSET

    net_average_order_value: OptionalNullable[MetricsTotalsNetAverageOrderValue] = UNSET

    average_revenue_per_user: OptionalNullable[MetricsTotalsAverageRevenuePerUser] = (
        UNSET
    )

    cost_per_user: OptionalNullable[MetricsTotalsCostPerUser] = UNSET

    active_user_by_event: OptionalNullable[MetricsTotalsActiveUserByEvent] = UNSET

    one_time_products: OptionalNullable[MetricsTotalsOneTimeProducts] = UNSET

    one_time_products_revenue: OptionalNullable[MetricsTotalsOneTimeProductsRevenue] = (
        UNSET
    )

    one_time_products_net_revenue: OptionalNullable[
        MetricsTotalsOneTimeProductsNetRevenue
    ] = UNSET

    new_subscriptions: OptionalNullable[MetricsTotalsNewSubscriptions] = UNSET

    new_subscriptions_revenue: OptionalNullable[
        MetricsTotalsNewSubscriptionsRevenue
    ] = UNSET

    new_subscriptions_net_revenue: OptionalNullable[
        MetricsTotalsNewSubscriptionsNetRevenue
    ] = UNSET

    renewed_subscriptions: OptionalNullable[MetricsTotalsRenewedSubscriptions] = UNSET

    renewed_subscriptions_revenue: OptionalNullable[
        MetricsTotalsRenewedSubscriptionsRevenue
    ] = UNSET

    renewed_subscriptions_net_revenue: OptionalNullable[
        MetricsTotalsRenewedSubscriptionsNetRevenue
    ] = UNSET

    active_subscriptions: OptionalNullable[MetricsTotalsActiveSubscriptions] = UNSET

    committed_subscriptions: OptionalNullable[MetricsTotalsCommittedSubscriptions] = (
        UNSET
    )

    monthly_recurring_revenue: OptionalNullable[
        MetricsTotalsMonthlyRecurringRevenue
    ] = UNSET

    committed_monthly_recurring_revenue: OptionalNullable[
        MetricsTotalsCommittedMonthlyRecurringRevenue
    ] = UNSET

    checkouts: OptionalNullable[MetricsTotalsCheckouts] = UNSET

    succeeded_checkouts: OptionalNullable[MetricsTotalsSucceededCheckouts] = UNSET

    checkouts_conversion: OptionalNullable[MetricsTotalsCheckoutsConversion] = UNSET

    canceled_subscriptions: OptionalNullable[MetricsTotalsCanceledSubscriptions] = UNSET

    canceled_subscriptions_customer_service: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsCustomerService
    ] = UNSET

    canceled_subscriptions_low_quality: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsLowQuality
    ] = UNSET

    canceled_subscriptions_missing_features: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsMissingFeatures
    ] = UNSET

    canceled_subscriptions_switched_service: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsSwitchedService
    ] = UNSET

    canceled_subscriptions_too_complex: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsTooComplex
    ] = UNSET

    canceled_subscriptions_too_expensive: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsTooExpensive
    ] = UNSET

    canceled_subscriptions_unused: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsUnused
    ] = UNSET

    canceled_subscriptions_other: OptionalNullable[
        MetricsTotalsCanceledSubscriptionsOther
    ] = UNSET

    churned_subscriptions: OptionalNullable[MetricsTotalsChurnedSubscriptions] = UNSET

    churn_rate: OptionalNullable[MetricsTotalsChurnRate] = UNSET

    ltv: OptionalNullable[MetricsTotalsLtv] = UNSET

    gross_margin: OptionalNullable[MetricsTotalsGrossMargin] = UNSET

    gross_margin_percentage: OptionalNullable[MetricsTotalsGrossMarginPercentage] = (
        UNSET
    )

    cashflow: OptionalNullable[MetricsTotalsCashflow] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        nullable_fields = [
            "orders",
            "revenue",
            "net_revenue",
            "cumulative_revenue",
            "net_cumulative_revenue",
            "costs",
            "cumulative_costs",
            "average_order_value",
            "net_average_order_value",
            "average_revenue_per_user",
            "cost_per_user",
            "active_user_by_event",
            "one_time_products",
            "one_time_products_revenue",
            "one_time_products_net_revenue",
            "new_subscriptions",
            "new_subscriptions_revenue",
            "new_subscriptions_net_revenue",
            "renewed_subscriptions",
            "renewed_subscriptions_revenue",
            "renewed_subscriptions_net_revenue",
            "active_subscriptions",
            "committed_subscriptions",
            "monthly_recurring_revenue",
            "committed_monthly_recurring_revenue",
            "checkouts",
            "succeeded_checkouts",
            "checkouts_conversion",
            "canceled_subscriptions",
            "canceled_subscriptions_customer_service",
            "canceled_subscriptions_low_quality",
            "canceled_subscriptions_missing_features",
            "canceled_subscriptions_switched_service",
            "canceled_subscriptions_too_complex",
            "canceled_subscriptions_too_expensive",
            "canceled_subscriptions_unused",
            "canceled_subscriptions_other",
            "churned_subscriptions",
            "churn_rate",
            "ltv",
            "gross_margin",
            "gross_margin_percentage",
            "cashflow",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .productpriceseattier import ProductPriceSeatTier, ProductPriceSeatTierTypedDict
from polar_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ProductPriceSeatTiersInputTypedDict(TypedDict):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTierTypedDict]
    r"""List of pricing tiers"""


class ProductPriceSeatTiersInput(BaseModel):
    r"""List of pricing tiers for seat-based pricing.

    The minimum and maximum seat limits are derived from the tiers:
    - minimum_seats = first tier's min_seats
    - maximum_seats = last tier's max_seats (None for unlimited)
    """

    tiers: List[ProductPriceSeatTier]
    r"""List of pricing tiers"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class WebhookEventType(str, Enum):
    CHECKOUT_CREATED = "checkout.created"
    CHECKOUT_UPDATED = "checkout.updated"
    CUSTOMER_CREATED = "customer.created"
    CUSTOMER_UPDATED = "customer.updated"
    CUSTOMER_DELETED = "customer.deleted"
    CUSTOMER_STATE_CHANGED = "customer.state_changed"
    CUSTOMER_SEAT_ASSIGNED = "customer_seat.assigned"
    CUSTOMER_SEAT_CLAIMED = "customer_seat.claimed"
    CUSTOMER_SEAT_REVOKED = "customer_seat.revoked"
    ORDER_CREATED = "order.created"
    ORDER_UPDATED = "order.updated"
    ORDER_PAID = "order.paid"
    ORDER_REFUNDED = "order.refunded"
    SUBSCRIPTION_CREATED = "subscription.created"
    SUBSCRIPTION_UPDATED = "subscription.updated"
    SUBSCRIPTION_ACTIVE = "subscription.active"
    SUBSCRIPTION_CANCELED = "subscription.canceled"
    SUBSCRIPTION_UNCANCELED = "subscription.uncanceled"
    SUBSCRIPTION_REVOKED = "subscription.revoked"
    SUBSCRIPTION_PAST_DUE = "subscription.past_due"
    REFUND_CREATED = "refund.created"
    REFUND_UPDATED = "refund.updated"
    PRODUCT_CREATED = "product.created"
    PRODUCT_UPDATED = "product.updated"
    BENEFIT_CREATED = "benefit.created"
    BENEFIT_UPDATED = "benefit.updated"
    BENEFIT_GRANT_CREATED = "benefit_grant.created"
    BENEFIT_GRANT_CYCLED = "benefit_grant.cycled"
    BENEFIT_GRANT_UPDATED = "benefit_grant.updated"
    BENEFIT_GRANT_REVOKED = "benefit_grant.revoked"
    ORGANIZATION_UPDATED = "organization.updated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing_extensions import TypedDict


class SeatClaimInfoTypedDict(TypedDict):
    r"""Read-only information about a seat claim invitation.
    Safe for email scanners - no side effects when fetched.
    """

    product_name: str
    r"""Name of the product"""
    product_id: str
    r"""ID of the product"""
    organization_name: str
    r"""Name of the organization"""
    organization_slug: str
    r"""Slug of the organization"""
    customer_email: str
    r"""Email of the customer assigned to this seat"""
    can_claim: bool
    r"""Whether the seat can be claimed"""


class SeatClaimInfo(BaseModel):
    r"""Read-only information about a seat claim invitation.
    Safe for email scanners - no side effects when fetched.
    """

    product_name: str
    r"""Name of the product"""

    product_id: str
    r"""ID of the product"""

    organization_name: str
    r"""Name of the organization"""

    organization_slug: str
    r"""Slug of the organization"""

    customer_email: str
    r"""Email of the customer assigned to this seat"""

    can_claim: bool
    r"""Whether the seat can be claimed"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .benefittype import BenefitType
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class BenefitGrantMetadataTypedDict(TypedDict):
    benefit_id: str
    benefit_grant_id: str
    benefit_type: BenefitType
    member_id: NotRequired[str]


class BenefitGrantMetadata(BaseModel):
    benefit_id: str

    benefit_grant_id: str

    benefit_type: BenefitType

    member_id: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from polar_sdk.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SubscriptionCanceledMetadataTypedDict(TypedDict):
    subscription_id: str
    amount: int
    currency: str
    recurring_interval: str
    recurring_interval_count: int
    canceled_at: str
    product_id: NotRequired[str]
    customer_cancellation_reason: NotRequired[str]
    customer_cancellation_comment: NotRequired[str]
    ends_at: NotRequired[str]
    cancel_at_period_end: NotRequired[bool]


class SubscriptionCanceledMetadata(BaseModel):
    subscription_id: str

    amount: int

    currency: str

    recurring_interval: str

    recurring_interval_count: int

    canceled_at: str

    product_id: Optional[str] = None

    customer_cancellation_reason: Optional[str] = None

    customer_cancellation_comment: Optional[str] = None

    ends_at: Optional[str] = None

    cancel_at_period_end: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OrganizationStatus(str, Enum):
    CREATED = "created"
    ONBOARDING_STARTED = "onboarding_started"
    INITIAL_REVIEW = "initial_review"
    ONGOING_REVIEW = "ongoing_review"
    DENIED = "denied"
    ACTIVE = "active"
